one_shot_rag_qa_docs = (
    """维基百科标题：The Last Horse\n《最后的马》（西班牙语：El último caballo）是1950年的西班牙喜剧电影，由埃德加·内维尔执导，费尔南多·费尔南·戈麦斯主演。\n"""
    """维基百科标题：Southampton\n南安普顿大学成立于1862年，1952年获得皇家宪章成为大学，现有超过22,000名学生。在2010年世界大学学术排名中，该校位列全球前100名研究型大学。2010年，泰晤士高等教育-QS世界大学排名将南安普顿大学列为全球前80名。该校自认为是英国前5名的研究型大学。南安普顿大学在工程科学、海洋学、化学、癌症科学、声振研究、计算机科学与电子、光电子学以及纺织品保护中心（将于2009年10月关闭）的纺织品保护领域享有全球声誉。它也是南安普顿国家海洋学中心（NOCS）的所在地，该中心是自然环境研究委员会资助的海洋研究核心。\n"""
    """维基百科标题：Stanton Township, Champaign County, Illinois\n斯坦顿镇是美国伊利诺伊州尚佩恩县的一个镇。根据2010年人口普查，其人口为505人，包含202个住房单元。\n"""
    """维基百科标题：Neville A. Stanton\n内维尔·A·斯坦顿是南安普顿大学的英国人为因素与工效学教授。斯坦顿教授是特许工程师（C.Eng）、特许心理学家（C.Psychol）和特许工效学家（C.ErgHF）。他撰写和编辑了超过40本书籍和300多篇同行评审期刊论文，探讨该学科的应用。斯坦顿是英国心理学会会员、工效学与人为因素学会会员以及工程技术学会会员。他的论文发表在包括《自然》在内的学术期刊上。他还帮助多家机构设计新的人机界面，如捷豹汽车的自适应巡航控制系统。\n"""
    """维基百科标题：Finding Nemo\n《海底总动员》影院版海报 导演：安德鲁·斯坦顿 制片人：格雷厄姆·沃尔特斯 编剧：安德鲁·斯坦顿、鲍勃·彼得森、大卫·雷诺兹 故事：安德鲁·斯坦顿 主演：艾伯特·布鲁克斯、艾伦·德詹尼丝、亚历山大·古尔德、威廉·达福 配乐：托马斯·纽曼 摄影：莎伦·卡拉汉、杰里米·拉斯基 剪辑：大卫·伊恩·索尔特 制作公司：华特迪士尼影片公司、皮克斯动画工作室 发行公司：博伟影业发行 上映日期：2003年5月30日 片长：100分钟 国家：美国 语言：英语 预算：9400万美元 票房：9.403亿美元"""
)



one_shot_ircot_demo = (
    f'{one_shot_rag_qa_docs}'
    '\n\n问题：'
    f"内维尔·A·斯坦顿的雇主成立于哪一年？"
    '\n思考：'
    f"内维尔·A·斯坦顿的雇主是南安普顿大学。南安普顿大学成立于1862年。因此答案是：1862年。"
    '\n\n'
)


rag_qa_system = (
    '作为高级阅读理解助手，你的任务是仔细分析文本段落和相应问题。'
    '你的回应从“思考：”开始，在此处你需要有条理地分解推理过程，说明你如何得出结论。'
    '最后以“答案：”结尾，呈现简洁、明确的回应，无需额外阐述。'
)

one_shot_rag_qa_input = (
    f"{one_shot_rag_qa_docs}"
    "\n\n问题："
    "内维尔·A·斯坦顿的雇主成立于哪一年？"
    '\n思考：'
)

one_shot_rag_qa_output = (
    "内维尔·A·斯坦顿的雇主是南安普顿大学。南安普顿大学成立于1862年。"
    "\n答案：1862年。"
)


prompt_template = [
    {"role": "system", "content": rag_qa_system},
    {"role": "user", "content": one_shot_rag_qa_input},
    {"role": "assistant", "content": one_shot_rag_qa_output},
    {"role": "user", "content": "${prompt_user}"}
]